@@ -83,6 +83,16 @@
  * An auxiliary class used by [class@ViewStack].
  */
 
+/**
+ * AdwViewStackPages:
+ *
+ * An auxiliary class used by [class@ViewStack].
+ *
+ * See [property@ViewStack:pages].
+ *
+ * Since: 1.4
+ */
+
 #define OPPOSITE_ORIENTATION(_orientation) (1 - (_orientation))
 
 enum {
@@ -135,6 +145,20 @@ enum {
 
 static GParamSpec *page_props[LAST_PAGE_PROP];
 
+struct _AdwViewStackPages
+{
+  GObject parent_instance;
+  AdwViewStack *stack;
+};
+
+enum {
+  PAGES_PROP_0,
+  PAGES_PROP_SELECTED_PAGE,
+  N_PAGES_PROPS,
+};
+
+static GParamSpec *pages_props[N_PAGES_PROPS];
+
 struct _AdwViewStack {
   GtkWidget parent_instance;
 
@@ -211,6 +235,9 @@ adw_view_stack_page_set_property (GObject      *object,
   switch (property_id) {
   case PAGE_PROP_CHILD:
     g_set_object (&self->widget, g_value_get_object (value));
+    if (self->widget)
+      gtk_accessible_set_accessible_parent (GTK_ACCESSIBLE (self->widget),
+                                            GTK_ACCESSIBLE (self), NULL);
     break;
   case PAGE_PROP_NAME:
     adw_view_stack_page_set_name (self, g_value_get_string (value));
@@ -262,10 +289,7 @@ adw_view_stack_page_finalize (GObject *object)
   g_clear_pointer (&self->name, g_free);
   g_clear_pointer (&self->title, g_free);
   g_clear_pointer (&self->icon_name, g_free);
-
-  if (self->last_focus)
-    g_object_remove_weak_pointer (G_OBJECT (self->last_focus),
-                                  (gpointer *) &self->last_focus);
+  g_clear_weak_pointer (&self->last_focus);
 
   G_OBJECT_CLASS (adw_view_stack_page_parent_class)->finalize (object);
 }
@@ -281,7 +305,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   object_class->set_property = adw_view_stack_page_set_property;
 
   /**
-   * AdwViewStackPage:child: (attributes org.gtk.Property.get=adw_view_stack_page_get_child)
+   * AdwViewStackPage:child:
    *
    * The stack child to which the page belongs.
    */
@@ -291,7 +315,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwViewStackPage:name: (attributes org.gtk.Property.get=adw_view_stack_page_get_name org.gtk.Property.set=adw_view_stack_page_set_name)
+   * AdwViewStackPage:name:
    *
    * The name of the child page.
    */
@@ -301,7 +325,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:title: (attributes org.gtk.Property.get=adw_view_stack_page_get_title org.gtk.Property.set=adw_view_stack_page_set_title)
+   * AdwViewStackPage:title:
    *
    * The title of the child page.
    */
@@ -311,7 +335,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
+   * AdwViewStackPage:use-underline:
    *
    * Whether an embedded underline in the title indicates a mnemonic.
    */
@@ -321,7 +345,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:icon-name: (attributes org.gtk.Property.get=adw_view_stack_page_get_icon_name org.gtk.Property.set=adw_view_stack_page_set_icon_name)
+   * AdwViewStackPage:icon-name:
    *
    * The icon name of the child page.
    */
@@ -331,7 +355,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:needs-attention: (attributes org.gtk.Property.get=adw_view_stack_page_get_needs_attention org.gtk.Property.set=adw_view_stack_page_set_needs_attention)
+   * AdwViewStackPage:needs-attention:
    *
    * Whether the page requires the user attention.
    *
@@ -343,7 +367,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:badge-number: (attributes org.gtk.Property.get=adw_view_stack_page_get_badge_number org.gtk.Property.set=adw_view_stack_page_set_badge_number)
+   * AdwViewStackPage:badge-number:
    *
    * The badge number for this page.
    *
@@ -358,7 +382,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:visible: (attributes org.gtk.Property.get=adw_view_stack_page_get_visible org.gtk.Property.set=adw_view_stack_page_set_visible)
+   * AdwViewStackPage:visible:
    *
    * Whether this page is visible.
    *
@@ -476,16 +500,6 @@ adw_view_stack_page_accessible_init (GtkAccessibleInterface *iface)
   iface->get_bounds = adw_view_stack_page_accessible_get_bounds;
 }
 
-#define ADW_TYPE_VIEW_STACK_PAGES (adw_view_stack_pages_get_type ())
-
-G_DECLARE_FINAL_TYPE (AdwViewStackPages, adw_view_stack_pages, ADW, VIEW_STACK_PAGES, GObject)
-
-struct _AdwViewStackPages
-{
-  GObject parent_instance;
-  AdwViewStack *stack;
-};
-
 static GType
 adw_view_stack_pages_get_item_type (GListModel *model)
 {
@@ -561,6 +575,102 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStackPages, adw_view_stack_pages, G_TYPE_O
                                G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, adw_view_stack_pages_list_model_init)
                                G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL, adw_view_stack_pages_selection_model_init))
 
+/**
+ * adw_view_stack_pages_get_selected_page:
+ * @self: a [class@ViewStackPages]
+ *
+ * Gets the [class@ViewStackPage] for the visible child of a view stack
+ *
+ * Gets the [class@ViewStackPage] for the visible child of the associated stack.
+ *
+ * Returns `NULL` if there's no selected page.
+ *
+ * Returns: (transfer none) (nullable): the stack page
+ *
+ * Since: 1.4
+ */
+AdwViewStackPage *
+adw_view_stack_pages_get_selected_page (AdwViewStackPages *self)
+{
+  GtkWidget *child;
+
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGES (self), NULL);
+
+  if (self->stack == NULL)
+    return NULL;
+
+  if ((child = adw_view_stack_get_visible_child (self->stack)))
+    return adw_view_stack_get_page (self->stack, child);
+
+  return NULL;
+}
+
+/**
+ * adw_view_stack_pages_set_selected_page:
+ * @self: a [class@ViewStackPages]
+ * @page: a stack page within the associated stack
+ *
+ * Sets the visible child in the associated [class@ViewStack].
+ *
+ * See [property@ViewStack:visible-child].
+ *
+ * Since: 1.4
+ */
+void
+adw_view_stack_pages_set_selected_page (AdwViewStackPages *self,
+                                        AdwViewStackPage  *page)
+{
+  GtkWidget *child = NULL;
+
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGES (self));
+  g_return_if_fail (!page || ADW_IS_VIEW_STACK_PAGE (page));
+
+  if (self->stack == NULL)
+    return;
+
+  if (page == adw_view_stack_pages_get_selected_page (self))
+    return;
+
+  if (page != NULL)
+    child = adw_view_stack_page_get_child (page);
+
+  adw_view_stack_set_visible_child (self->stack, child);
+}
+
+static void
+adw_view_stack_pages_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+  AdwViewStackPages *self = ADW_VIEW_STACK_PAGES (object);
+
+  switch (prop_id) {
+  case PAGES_PROP_SELECTED_PAGE:
+    g_value_set_object (value, adw_view_stack_pages_get_selected_page (self));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
+static void
+adw_view_stack_pages_set_property (GObject      *object,
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
+{
+  AdwViewStackPages *self = ADW_VIEW_STACK_PAGES (object);
+
+  switch (prop_id) {
+  case PAGES_PROP_SELECTED_PAGE:
+    adw_view_stack_pages_set_selected_page (self, g_value_get_object (value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
 static void
 adw_view_stack_pages_init (AdwViewStackPages *pages)
 {
@@ -569,6 +679,29 @@ adw_view_stack_pages_init (AdwViewStackPages *pages)
 static void
 adw_view_stack_pages_class_init (AdwViewStackPagesClass *class)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  object_class->get_property = adw_view_stack_pages_get_property;
+  object_class->set_property = adw_view_stack_pages_set_property;
+
+  /**
+   * AdwViewStackPages:selected-page:
+   *
+   * The selected [class@ViewStackPage] within the [class@ViewStackPages].
+   *
+   * This can be used to keep an up-to-date view of the [class@ViewStackPage] for
+   * The visible [class@ViewStackPage] within the associated [class@ViewStackPages].
+   *
+   * This can be used to keep an up-to-date view of the visible child.
+   *
+   * Since: 1.4
+   */
+  pages_props[PAGES_PROP_SELECTED_PAGE] =
+    g_param_spec_object ("selected-page", NULL, NULL,
+                         ADW_TYPE_VIEW_STACK_PAGE,
+                         (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, N_PAGES_PROPS, pages_props);
 }
 
 static AdwViewStackPages *
@@ -688,12 +821,7 @@ set_visible_child (AdwViewStack     *self,
       gtk_widget_is_ancestor (focus, self->visible_child->widget)) {
     contains_focus = TRUE;
 
-    if (self->visible_child->last_focus)
-      g_object_remove_weak_pointer (G_OBJECT (self->visible_child->last_focus),
-                                    (gpointer *)&self->visible_child->last_focus);
-    self->visible_child->last_focus = focus;
-    g_object_add_weak_pointer (G_OBJECT (self->visible_child->last_focus),
-                               (gpointer *)&self->visible_child->last_focus);
+    g_set_weak_pointer (&self->visible_child->last_focus, focus);
   }
 
   if (self->visible_child && self->visible_child->widget)
@@ -720,6 +848,9 @@ set_visible_child (AdwViewStack     *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_CHILD]);
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_CHILD_NAME]);
 
+  if (self->pages != NULL)
+    g_object_notify_by_pspec (G_OBJECT (self->pages), pages_props[PAGES_PROP_SELECTED_PAGE]);
+
   if (self->pages) {
     if (old_pos == GTK_INVALID_LIST_POSITION && new_pos == GTK_INVALID_LIST_POSITION)
       ; /* nothing to do */
@@ -1016,9 +1147,7 @@ adw_view_stack_finalize (GObject *object)
 {
   AdwViewStack *self = ADW_VIEW_STACK (object);
 
-  if (self->pages)
-    g_object_remove_weak_pointer (G_OBJECT (self->pages),
-                                  (gpointer *) &self->pages);
+  g_clear_weak_pointer (&self->pages);
 
   G_OBJECT_CLASS (adw_view_stack_parent_class)->finalize (object);
 }
@@ -1040,7 +1169,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwViewStack:hhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_hhomogeneous org.gtk.Property.set=adw_view_stack_set_hhomogeneous)
+   * AdwViewStack:hhomogeneous:
    *
    * Whether the stack is horizontally homogeneous.
    *
@@ -1056,7 +1185,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:vhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_vhomogeneous org.gtk.Property.set=adw_view_stack_set_vhomogeneous)
+   * AdwViewStack:vhomogeneous:
    *
    * Whether the stack is vertically homogeneous.
    *
@@ -1072,7 +1201,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:visible-child: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child org.gtk.Property.set=adw_view_stack_set_visible_child)
+   * AdwViewStack:visible-child:
    *
    * The widget currently visible in the stack.
    */
@@ -1082,7 +1211,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:visible-child-name: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child_name org.gtk.Property.set=adw_view_stack_set_visible_child_name)
+   * AdwViewStack:visible-child-name:
    *
    * The name of the widget currently visible in the stack.
    *
@@ -1094,7 +1223,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:pages: (attributes org.gtk.Property.get=adw_view_stack_get_pages)
+   * AdwViewStack:pages:
    *
    * A selection model with the stack's pages.
    *
@@ -1162,7 +1291,7 @@ adw_view_stack_accessible_init (GtkAccessibleInterface *iface)
 }
 
 /**
- * adw_view_stack_page_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_view_stack_page_get_child:
  * @self: a view stack page
  *
  * Gets the stack child to which @self belongs.
@@ -1178,7 +1307,7 @@ adw_view_stack_page_get_child (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_get_name: (attributes org.gtk.Method.get_property=name)
+ * adw_view_stack_page_get_name:
  * @self: a view stack page
  *
  * Gets the name of the page.
@@ -1194,7 +1323,7 @@ adw_view_stack_page_get_name (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_name: (attributes org.gtk.Method.set_property=name)
+ * adw_view_stack_page_set_name:
  * @self: a view stack page
  * @name: (nullable): the page name
  *
@@ -1228,12 +1357,9 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
     }
   }
 
-  if (name == self->name)
+  if (!g_set_str (&self->name, name))
     return;
 
-  g_free (self->name);
-  self->name = g_strdup (name);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NAME]);
 
   if (stack && stack->visible_child == self)
@@ -1242,7 +1368,7 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_title: (attributes org.gtk.Method.get_property=title)
+ * adw_view_stack_page_get_title:
  * @self: a view stack page
  *
  * Gets the page title.
@@ -1258,7 +1384,7 @@ adw_view_stack_page_get_title (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_title: (attributes org.gtk.Method.set_property=title)
+ * adw_view_stack_page_set_title:
  * @self: a view stack page
  * @title: (nullable): the page title
  *
@@ -1270,12 +1396,9 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
 {
   g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
 
-  if (title == self->title)
+  if (!g_set_str (&self->title, title))
     return;
 
-  g_free (self->title);
-  self->title = g_strdup (title);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_TITLE]);
 
   gtk_accessible_update_property (GTK_ACCESSIBLE (self),
@@ -1284,7 +1407,7 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * adw_view_stack_page_get_use_underline:
  * @self: a view stack page
  *
  * Gets whether underlines in the page title indicate mnemonics.
@@ -1298,7 +1421,7 @@ adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * adw_view_stack_page_set_use_underline:
  * @self: a view stack page
  * @use_underline: the new value to set
  *
@@ -1319,7 +1442,7 @@ adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_view_stack_page_get_icon_name:
  * @self: a view stack page
  *
  * Gets the icon name of the page.
@@ -1335,7 +1458,7 @@ adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_view_stack_page_set_icon_name:
  * @self: a view stack page
  * @icon_name: (nullable): the icon name
  *
@@ -1347,17 +1470,14 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
 {
   g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
 
-  if (icon_name == self->icon_name)
+  if (!g_set_str (&self->icon_name, icon_name))
     return;
 
-  g_free (self->icon_name);
-  self->icon_name = g_strdup (icon_name);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_ICON_NAME]);
 }
 
 /**
- * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
+ * adw_view_stack_page_get_needs_attention:
  * @self: a view stack page
  *
  * Gets whether the page requires the user attention.
@@ -1373,7 +1493,7 @@ adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
+ * adw_view_stack_page_set_needs_attention:
  * @self: a view stack page
  * @needs_attention: the new value to set
  *
@@ -1398,7 +1518,7 @@ adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
+ * adw_view_stack_page_get_badge_number:
  * @self: a view stack page
  *
  * Gets the badge number for this page.
@@ -1414,7 +1534,7 @@ adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
+ * adw_view_stack_page_set_badge_number:
  * @self: a view stack page
  * @badge_number: the new value to set
  *
@@ -1440,7 +1560,7 @@ adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
+ * adw_view_stack_page_get_visible:
  * @self: a view stack page
  *
  * Gets whether @self is visible in its `AdwViewStack`.
@@ -1459,7 +1579,7 @@ adw_view_stack_page_get_visible (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
+ * adw_view_stack_page_set_visible:
  * @self: a view stack page
  * @visible: whether @self is visible
  *
@@ -1515,6 +1635,7 @@ adw_view_stack_add (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, NULL, NULL, NULL);
 }
@@ -1538,6 +1659,7 @@ adw_view_stack_add_named (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, NULL, NULL);
 }
@@ -1564,6 +1686,7 @@ adw_view_stack_add_titled (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, title, NULL);
 }
@@ -1594,6 +1717,7 @@ adw_view_stack_add_titled_with_icon (AdwViewStack *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, title, icon_name);
 }
@@ -1672,10 +1796,10 @@ adw_view_stack_get_child_by_name (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_visible_child: (attributes org.gtk.Method.get_property=visible-child)
+ * adw_view_stack_get_visible_child:
  * @self: a view stack
  *
- * Gets the currently visible child of @self, .
+ * Gets the currently visible child of @self.
  *
  * Returns: (transfer none) (nullable): the visible child
  */
@@ -1688,7 +1812,7 @@ adw_view_stack_get_visible_child (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_visible_child: (attributes org.gtk.Method.set_property=visible-child)
+ * adw_view_stack_set_visible_child:
  * @self: a view stack
  * @child: a child of @self
  *
@@ -1716,7 +1840,7 @@ adw_view_stack_set_visible_child (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_visible_child_name: (attributes org.gtk.Method.get_property=visible-child-name)
+ * adw_view_stack_get_visible_child_name:
  * @self: a view stack
  *
  * Returns the name of the currently visible child of @self.
@@ -1732,7 +1856,7 @@ adw_view_stack_get_visible_child_name (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_visible_child_name: (attributes org.gtk.Method.set_property=visible-child-name)
+ * adw_view_stack_set_visible_child_name:
  * @self: a view stack
  * @name: the name of the child
  *
@@ -1764,7 +1888,7 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * adw_view_stack_get_hhomogeneous:
  * @self: a view stack
  *
  * Gets whether @self is horizontally homogeneous.
@@ -1780,7 +1904,7 @@ adw_view_stack_get_hhomogeneous (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_hhomogeneous: (attributes org.gtk.Method.set_property=hhomogeneous)
+ * adw_view_stack_set_hhomogeneous:
  * @self: a view stack
  * @hhomogeneous: whether to make @self horizontally homogeneous
  *
@@ -1812,7 +1936,7 @@ adw_view_stack_set_hhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
+ * adw_view_stack_get_vhomogeneous:
  * @self: a view stack
  *
  * Gets whether @self is vertically homogeneous.
@@ -1828,7 +1952,7 @@ adw_view_stack_get_vhomogeneous (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_vhomogeneous: (attributes org.gtk.Method.set_property=vhomogeneous)
+ * adw_view_stack_set_vhomogeneous:
  * @self: a view stack
  * @vhomogeneous: whether to make @self vertically homogeneous
  *
@@ -1860,7 +1984,7 @@ adw_view_stack_set_vhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_pages: (attributes org.gtk.Method.get_property=pages)
+ * adw_view_stack_get_pages:
  * @self: a view stack
  *
  * Returns a [iface@Gio.ListModel] that contains the pages of the stack.
@@ -1879,8 +2003,8 @@ adw_view_stack_get_pages (AdwViewStack *self)
   if (self->pages)
     return g_object_ref (self->pages);
 
-  self->pages = GTK_SELECTION_MODEL (adw_view_stack_pages_new (self));
-  g_object_add_weak_pointer (G_OBJECT (self->pages), (gpointer *) &self->pages);
+  g_set_weak_pointer (&self->pages,
+                      GTK_SELECTION_MODEL (adw_view_stack_pages_new (self)));
 
   return self->pages;
 }
